@@ -29,7 +29,7 @@
 #include <pwd.h>
 #endif /* HAVE_PWD_H */
 #else /* ! HAVE_GETPASS */
-#ifndef __WIN__
+#ifndef _WIN32
 #include <sys/ioctl.h>
 #ifdef HAVE_TERMIOS_H				/* For tty-password */
 #include	<termios.h>
@@ -49,22 +49,22 @@
 #endif
 #else
 #include <conio.h>
-#endif /* __WIN__ */
+#endif /* _WIN32 */
 #endif /* HAVE_GETPASS */
 
 #ifdef HAVE_GETPASSPHRASE			/* For Solaris */
 #define getpass(A) getpassphrase(A)
 #endif
 
-#ifdef __WIN__
+#ifdef _WIN32
 /* were just going to fake it here and get input from
    the keyboard */
 
-char *get_tty_password(const char *opt_message)
+char *my_get_tty_password(const char *opt_message)
 {
   char to[80];
   char *pos=to,*end=to+sizeof(to)-1;
-  DBUG_ENTER("get_tty_password");
+  DBUG_ENTER("my_get_tty_password");
   _cputs(opt_message ? opt_message : "Enter password: ");
   for (;;)
   {
@@ -145,7 +145,7 @@ static void get_password(char *to,uint length,int fd, my_bool echo)
 #endif /* ! HAVE_GETPASS */
 
 
-char *get_tty_password(const char *opt_message)
+char *my_get_tty_password(const char *opt_message)
 {
 #ifdef HAVE_GETPASS
   char *passbuff;
@@ -154,7 +154,7 @@ char *get_tty_password(const char *opt_message)
 #endif /* HAVE_GETPASS */
   char buff[80];
 
-  DBUG_ENTER("get_tty_password");
+  DBUG_ENTER("my_get_tty_password");
 
 #ifdef HAVE_GETPASS
   passbuff = getpass(opt_message ? opt_message : "Enter password: ");
@@ -204,4 +204,4 @@ char *get_tty_password(const char *opt_message)
   DBUG_RETURN(my_strdup(PSI_INSTRUMENT_ME, buff, MYF(MY_FAE)));
 }
 
-#endif /*__WIN__*/
+#endif /*_WIN32*/
