@@ -133,6 +133,8 @@ double getopt_ulonglong2double(ulonglong v)
   return u.dbl;
 }
 
+#define SET_HO_ERROR_AND_CONTINUE(e) { ho_error= (e); (*argc)--; continue; }
+
 /**
   Handle command line options.
   Sort options.
@@ -202,7 +204,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
   const char *UNINIT_VAR(prev_found);
   const struct my_option *optp;
   void *value;
-  int error, i;
+  int ho_error= 0, error, i;
   my_bool is_cmdline_arg= 1;
   DBUG_ENTER("handle_options");
 
@@ -216,7 +218,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
 
   is_cmdline_arg= !is_file_marker(**argv);
 
-  for (pos= *argv, pos_end=pos+ *argc; pos != pos_end ; pos++)
+  for (pos= *argv, pos_end=pos+ *argc; pos < pos_end ; pos++)
   {
     char **first= pos;
     char *cur_arg= *pos;
@@ -305,7 +307,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                                                my_progname, special_opt_prefix[i],
                                                opt_str, special_opt_prefix[i],
                                                prev_found);
-		    DBUG_RETURN(EXIT_AMBIGUOUS_OPTION);
+		    SET_HO_ERROR_AND_CONTINUE(EXIT_AMBIGUOUS_OPTION)
 		  }
 		  switch (i) {
 		  case OPT_SKIP:
@@ -350,7 +352,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                                          "%s: unknown variable '%s'",
                                          my_progname, cur_arg);
 	      if (!option_is_loose)
-		DBUG_RETURN(EXIT_UNKNOWN_VARIABLE);
+		SET_HO_ERROR_AND_CONTINUE(EXIT_UNKNOWN_VARIABLE)
 	    }
 	    else
 	    {
@@ -360,7 +362,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                                          "%s: unknown option '--%s'", 
                                          my_progname, cur_arg);
 	      if (!option_is_loose)
-		DBUG_RETURN(EXIT_UNKNOWN_OPTION);
+		SET_HO_ERROR_AND_CONTINUE(EXIT_UNKNOWN_OPTION)
 	    }
 	    if (option_is_loose)
 	    {
@@ -377,7 +379,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
               my_getopt_error_reporter(ERROR_LEVEL,
                                        "%s: variable prefix '%s' is not unique",
                                        my_progname, opt_str);
-	    DBUG_RETURN(EXIT_VAR_PREFIX_NOT_UNIQUE);
+	    SET_HO_ERROR_AND_CONTINUE(EXIT_VAR_PREFIX_NOT_UNIQUE)
 	  }
 	  else
 	  {
@@ -386,7 +388,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                                        "%s: ambiguous option '--%s' (%s, %s)",
                                        my_progname, opt_str, prev_found, 
                                        optp->name);
-	    DBUG_RETURN(EXIT_AMBIGUOUS_OPTION);
+	    SET_HO_ERROR_AND_CONTINUE(EXIT_AMBIGUOUS_OPTION)
 	  }
 	}
 	if ((optp->var_type & GET_TYPE_MASK) == GET_DISABLED)
@@ -400,14 +402,14 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
 	    (*argc)--;
 	    continue;
 	  }
-	  DBUG_RETURN(EXIT_OPTION_DISABLED);
+	  SET_HO_ERROR_AND_CONTINUE(EXIT_OPTION_DISABLED)
 	}
         error= 0;
 	value= optp->var_type & GET_ASK_ADDR
           ? (*my_getopt_get_addr)(key_name, (uint)strlen(key_name), optp, &error)
           : optp->value;
         if (error)
-          DBUG_RETURN(error);
+          SET_HO_ERROR_AND_CONTINUE(error)
 
 	if (optp->arg_type == NO_ARG)
 	{
@@ -422,7 +424,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
               my_getopt_error_reporter(ERROR_LEVEL,
                                        "%s: option '--%s' cannot take an argument",
                                        my_progname, optp->name);
-	    DBUG_RETURN(EXIT_NO_ARGUMENT_ALLOWED);
+	    SET_HO_ERROR_AND_CONTINUE(EXIT_NO_ARGUMENT_ALLOWED)
 	  }
 	  if ((optp->var_type & GET_TYPE_MASK) == GET_BOOL)
 	  {
@@ -451,7 +453,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
             if (get_one_option(optp, *((my_bool*) value) ?
                                enabled_my_option : disabled_my_option,
                                filename))
-              DBUG_RETURN(EXIT_ARGUMENT_INVALID);
+              SET_HO_ERROR_AND_CONTINUE(EXIT_ARGUMENT_INVALID)
 	    continue;
 	  }
 	  argument= optend;
@@ -465,7 +467,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                                      "option '--%s' cannot take an argument",
                                      my_progname, optp->name);
 
-	    DBUG_RETURN(EXIT_NO_ARGUMENT_ALLOWED);
+	    SET_HO_ERROR_AND_CONTINUE(EXIT_NO_ARGUMENT_ALLOWED)
 	  }
 	  if (!(optp->var_type & GET_AUTO))
 	  {
@@ -475,7 +477,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
 				     "unsupported by option '--%s'",
                                      my_progname, optp->name);
 	    if (!option_is_loose)
-	      DBUG_RETURN(EXIT_ARGUMENT_INVALID);
+	      SET_HO_ERROR_AND_CONTINUE(EXIT_ARGUMENT_INVALID)
 	    continue;
 	  }
 	  else
@@ -494,7 +496,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
               my_getopt_error_reporter(ERROR_LEVEL,
                                        "%s: option '--%s' requires an argument",
                                        my_progname, optp->name);
-	    DBUG_RETURN(EXIT_ARGUMENT_REQUIRED);
+	    SET_HO_ERROR_AND_CONTINUE(EXIT_ARGUMENT_REQUIRED)
 	  }
 	  argument= *pos;
 	  (*argc)--;
@@ -519,14 +521,14 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
 		  fprintf(stderr,
 			  "%s: ERROR: Option '-%c' used, but is disabled\n",
 			  my_progname, optp->id);
-		DBUG_RETURN(EXIT_OPTION_DISABLED);
+		SET_HO_ERROR_AND_CONTINUE(EXIT_OPTION_DISABLED)
 	      }
 	      if ((optp->var_type & GET_TYPE_MASK) == GET_BOOL &&
 		  optp->arg_type == NO_ARG)
 	      {
 		*((my_bool*) optp->value)= (my_bool) 1;
                 if (get_one_option(optp, argument, filename))
-                  DBUG_RETURN(EXIT_UNSPECIFIED_ERROR);
+                  SET_HO_ERROR_AND_CONTINUE(EXIT_UNSPECIFIED_ERROR)
 		continue;
 	      }
 	      else if (optp->arg_type == REQUIRED_ARG ||
@@ -546,7 +548,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                     if (optp->var_type == GET_BOOL)
                       *((my_bool*) optp->value)= (my_bool) 1;
                     if (get_one_option(optp, argument, filename))
-                      DBUG_RETURN(EXIT_UNSPECIFIED_ERROR);
+                      SET_HO_ERROR_AND_CONTINUE(EXIT_UNSPECIFIED_ERROR)
                     continue;
                   }
 		  /* Check if there are more arguments after this one */
@@ -556,7 +558,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                       my_getopt_error_reporter(ERROR_LEVEL,
                                                "%s: option '-%c' requires an argument",
                                                my_progname, optp->id);
-                    DBUG_RETURN(EXIT_ARGUMENT_REQUIRED);
+                    SET_HO_ERROR_AND_CONTINUE(EXIT_ARGUMENT_REQUIRED)
 		  }
 		  argument= *++pos;
 		  (*argc)--;
@@ -565,9 +567,9 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
 	      }
 	      if ((error= setval(optp, optp->value, argument,
 				 set_maximum_value)))
-		DBUG_RETURN(error);
+		SET_HO_ERROR_AND_CONTINUE(error)
               if (get_one_option(optp, argument, filename))
-                DBUG_RETURN(EXIT_UNSPECIFIED_ERROR);
+                SET_HO_ERROR_AND_CONTINUE(EXIT_UNSPECIFIED_ERROR)
 	      break;
 	    }
 	  }
@@ -601,7 +603,7 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                 my_getopt_error_reporter(ERROR_LEVEL,
                                          "%s: unknown option '-%c'",
                                          my_progname, *optend);
-              DBUG_RETURN(EXIT_UNKNOWN_OPTION);
+              SET_HO_ERROR_AND_CONTINUE(EXIT_UNKNOWN_OPTION)
             }
 	  }
 	}
@@ -612,15 +614,17 @@ int handle_options(int *argc, char ***argv, const struct my_option *longopts,
       if ((!option_is_autoset) &&
 	  ((error= setval(optp, value, argument, set_maximum_value))) &&
           !option_is_loose)
-	DBUG_RETURN(error);
+	SET_HO_ERROR_AND_CONTINUE(error)
       if (get_one_option(optp, argument, filename))
-        DBUG_RETURN(EXIT_UNSPECIFIED_ERROR);
+        SET_HO_ERROR_AND_CONTINUE(EXIT_UNSPECIFIED_ERROR)
 
       (*argc)--; /* option handled (long), decrease argument count */
     }
     else /* non-option found */
       (*argv)[argvpos++]= cur_arg;
   }
+  if (ho_error)
+    DBUG_RETURN(ho_error);
   /*
     Destroy the first, already handled option, so that programs that look
     for arguments in 'argv', without checking 'argc', know when to stop.
@@ -860,7 +864,7 @@ static int setval(const struct my_option *opts, void *value, char *argument,
     }
     if (err)
     {
-      res= EXIT_UNKNOWN_SUFFIX;
+      res= err;
       goto ret;
     };
   }
@@ -994,7 +998,7 @@ static inline ulonglong eval_num_suffix(char *suffix, int *error)
   case 'E':
     return 1ULL << 60;
   default:
-    *error= 1;
+    *error= EXIT_UNKNOWN_SUFFIX;
     return 0ULL;
   }
 }
@@ -1020,15 +1024,18 @@ static longlong eval_num_suffix_ll(char *argument,
   if (errno == ERANGE)
   {
     my_getopt_error_reporter(ERROR_LEVEL,
-                             "Incorrect integer value: '%s'", argument);
-    *error= 1;
+                             "Integer value out of range for int64:"
+                             " '%s' for %s",
+                             argument, option_name);
+    *error= EXIT_ARGUMENT_INVALID;
     DBUG_RETURN(0);
   }
   num*= eval_num_suffix(endchar, error);
   if (*error)
-    fprintf(stderr,
-	    "Unknown suffix '%c' used for variable '%s' (value '%s')\n",
-	    *endchar, option_name, argument);
+    my_getopt_error_reporter(ERROR_LEVEL,
+                             "Unknown suffix '%c' used for variable '%s' (value '%s'). "
+                             "Legal suffix characters are: K, M, G, T, P, E",
+                             *endchar, option_name, argument);
   DBUG_RETURN(num);
 }
 
@@ -1051,7 +1058,7 @@ static ulonglong eval_num_suffix_ull(char *argument,
     my_getopt_error_reporter(ERROR_LEVEL,
                              "Incorrect unsigned value: '%s' for %s",
                              argument, option_name);
-    *error= 1;
+    *error= EXIT_ARGUMENT_INVALID;
     DBUG_RETURN(0);
   }
   *error= 0;
@@ -1060,15 +1067,18 @@ static ulonglong eval_num_suffix_ull(char *argument,
   if (errno == ERANGE)
   {
     my_getopt_error_reporter(ERROR_LEVEL,
-                             "Incorrect integer value: '%s' for %s",
+                             "Integer value out of range for uint64:"
+                             " '%s' for %s",
                              argument, option_name);
-    *error= 1;
+    *error= EXIT_ARGUMENT_INVALID;
     DBUG_RETURN(0);
   }
   num*= eval_num_suffix(endchar, error);
   if (*error)
     my_getopt_error_reporter(ERROR_LEVEL,
-                             "Unknown suffix '%c' used for variable '%s' (value '%s')",
+                             "Unknown suffix '%c' used for variable '%s'"
+                             " (value '%s')."
+                             " Legal suffix characters are: K, M, G, T, P, E",
                              *endchar, option_name, argument);
   DBUG_RETURN(num);
 }
@@ -1088,6 +1098,8 @@ static ulonglong eval_num_suffix_ull(char *argument,
 static longlong getopt_ll(char *arg, const struct my_option *optp, int *err)
 {
   longlong num=eval_num_suffix_ll(arg, err, (char*) optp->name);
+  if (*err)
+    return(0);
   return getopt_ll_limit_value(num, optp, NULL);
 }
 
@@ -1165,6 +1177,8 @@ longlong getopt_ll_limit_value(longlong num, const struct my_option *optp,
 static ulonglong getopt_ull(char *arg, const struct my_option *optp, int *err)
 {
   ulonglong num= eval_num_suffix_ull(arg, err, (char*) optp->name);
+  if (*err)
+    return(0);
   return getopt_ull_limit_value(num, optp, NULL);
 }
 
