@@ -704,15 +704,15 @@ public:
     Aggregator(sum), table(NULL), tmp_table_param(NULL), tree(NULL),
     always_null(false), use_distinct_values(false) {}
   virtual ~Aggregator_distinct ();
-  Aggregator_type Aggrtype() { return DISTINCT_AGGREGATOR; }
+  Aggregator_type Aggrtype() override { return DISTINCT_AGGREGATOR; }
 
-  bool setup(THD *);
-  void clear(); 
-  bool add();
-  void endup();
-  virtual my_decimal *arg_val_decimal(my_decimal * value);
-  virtual double arg_val_real();
-  virtual bool arg_is_null(bool use_null_value);
+  bool setup(THD *) override;
+  void clear() override; 
+  bool add() override;
+  void endup() override;
+  my_decimal *arg_val_decimal(my_decimal * value) override;
+  double arg_val_real() override;
+  bool arg_is_null(bool use_null_value) override;
 
   bool unique_walk_function(void *element);
   bool unique_walk_function_for_count(void *element);
@@ -731,15 +731,15 @@ public:
 
   Aggregator_simple (Item_sum *sum) :
     Aggregator(sum) {}
-  Aggregator_type Aggrtype() { return Aggregator::SIMPLE_AGGREGATOR; }
+  Aggregator_type Aggrtype() override { return Aggregator::SIMPLE_AGGREGATOR; }
 
-  bool setup(THD * thd) { return item_sum->setup(thd); }
-  void clear() { item_sum->clear(); }
-  bool add() { return item_sum->add(); }
-  void endup() {};
-  virtual my_decimal *arg_val_decimal(my_decimal * value);
-  virtual double arg_val_real();
-  virtual bool arg_is_null(bool use_null_value);
+  bool setup(THD * thd) override { return item_sum->setup(thd); }
+  void clear() override { item_sum->clear(); }
+  bool add() override { return item_sum->add(); }
+  void endup() override {};
+  my_decimal *arg_val_decimal(my_decimal * value) override;
+  double arg_val_real() override;
+  bool arg_is_null(bool use_null_value) override;
 };
 
 
@@ -755,7 +755,7 @@ public:
     Item_sum(thd, list) {}
   Item_sum_num(THD *thd, Item_sum_num *item):
     Item_sum(thd, item) {}
-  bool fix_fields(THD *, Item **);
+  bool fix_fields(THD *, Item **) override;
 };
 
 
@@ -807,7 +807,8 @@ public:
     max_length=21;
     base_flags&= ~item_base_t::MAYBE_NULL;
     null_value=0;
-    return FALSE; }
+    return false;
+  }
 };
 
 
@@ -834,8 +835,8 @@ public:
   }
   Item_sum_sum(THD *thd, Item_sum_sum *item);
   enum Sumfunctype sum_func() const override
-  { 
-    return has_with_distinct() ? SUM_DISTINCT_FUNC : SUM_FUNC; 
+  {
+    return has_with_distinct() ? SUM_DISTINCT_FUNC : SUM_FUNC;
   }
   void cleanup() override;
   void direct_add(my_decimal *add_sum_decimal);
@@ -865,7 +866,7 @@ public:
   }
   Item *copy_or_same(THD* thd) override;
   void remove() override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_sum>(thd, this); }
 
   bool supports_removal() const override
@@ -940,7 +941,7 @@ public:
     return has_with_distinct() ? name_distinct : name_normal;
   }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_count>(thd, this); }
 
   bool supports_removal() const override
@@ -998,7 +999,7 @@ public:
     count= 0;
     Item_sum_sum::cleanup();
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_avg>(thd, this); }
 
   bool supports_removal() const override
@@ -1087,7 +1088,7 @@ public:
     m_stddev= Stddev();
     Item_sum_double::cleanup();
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_variance>(thd, this); }
 };
 
@@ -1113,7 +1114,7 @@ class Item_sum_std final :public Item_sum_variance
     return sample ? stddev_samp_name : std_name;
   }
   Item *copy_or_same(THD* thd) override final;
-  Item *get_copy(THD *thd) override final
+  Item *do_get_copy(THD *thd) const override final
   { return get_item_copy<Item_sum_std>(thd, this); }
 };
 
@@ -1184,7 +1185,8 @@ public:
   {
     return get_arg(0)->real_type_handler();
   }
-  const TYPELIB *get_typelib() const  override { return args[0]->get_typelib(); }
+  const TYPELIB *get_typelib() const override
+  { return args[0]->get_typelib(); }
   void update_field() override;
   void min_max_update_str_field();
   void min_max_update_real_field();
@@ -1215,7 +1217,7 @@ public:
     return sum_name;
   }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_min>(thd, this); }
 };
 
@@ -1225,7 +1227,7 @@ class Item_sum_max final :public Item_sum_min_max
 public:
   Item_sum_max(THD *thd, Item *item_par): Item_sum_min_max(thd, item_par, -1) {}
   Item_sum_max(THD *thd, Item_sum_max *item) :Item_sum_min_max(thd, item) {}
-  enum Sumfunctype sum_func () const  override {return MAX_FUNC;}
+  enum Sumfunctype sum_func() const override {return MAX_FUNC;}
 
   bool add() override;
   LEX_CSTRING func_name_cstring() const override
@@ -1234,7 +1236,7 @@ public:
     return sum_name;
   }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_max>(thd, this); }
 };
 
@@ -1276,9 +1278,7 @@ public:
       clear_as_window();
     Item_sum_int::cleanup();
   }
-  void setup_window_func(THD *thd __attribute__((unused)),
-                         Window_spec *window_spec __attribute__((unused)))
-    override
+  void setup_window_func(THD *, Window_spec *) override
   {
     as_window_function= TRUE;
     clear_as_window();
@@ -1329,7 +1329,7 @@ public:
     return sum_name;
   }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_or>(thd, this); }
 
 private:
@@ -1350,7 +1350,7 @@ public:
     return sum_min_name;
   }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_and>(thd, this); }
 
 private:
@@ -1369,7 +1369,7 @@ public:
     return sum_min_name;
   }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_xor>(thd, this); }
 
 private:
@@ -1513,7 +1513,7 @@ public:
   {
     return sp_result_field;
   }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_sp>(thd, this); }
   Item *copy_or_same(THD *thd) override;
 };
@@ -1579,8 +1579,9 @@ public:
   String *val_str(String *str) override
   { return val_string_from_real(str); }
   double val_real() override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_avg_field_double>(thd, this); }
+  Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
 };
 
 
@@ -1609,8 +1610,9 @@ public:
     return VDec(this).to_string_round(str, decimals);
   }
   my_decimal *val_decimal(my_decimal *) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_avg_field_decimal>(thd, this); }
+  Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
 };
 
 
@@ -1631,8 +1633,9 @@ public:
   bool is_null() override { update_null_value(); return null_value; }
   const Type_handler *type_handler() const override
   { return &type_handler_double; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_variance_field>(thd, this); }
+  Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
 };
 
 
@@ -1644,8 +1647,9 @@ public:
   { }
   enum Type type() const override { return FIELD_STD_ITEM; }
   double val_real() override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_std_field>(thd, this); }
+  Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
 };
 
 
@@ -1743,7 +1747,7 @@ class Item_sum_udf_float :public Item_udf_sum
   bool fix_length_and_dec() override
   { fix_num_length_and_dec(); return FALSE; }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_udf_float>(thd, this); }
 };
 
@@ -1770,7 +1774,7 @@ public:
   }
   bool fix_length_and_dec() override { decimals=0; max_length=21; return FALSE; }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_udf_int>(thd, this); }
 };
 
@@ -1812,7 +1816,7 @@ public:
   { return string_type_handler(); }
   bool fix_length_and_dec() override;
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_udf_str>(thd, this); }
 };
 
@@ -1844,7 +1848,7 @@ public:
   bool fix_length_and_dec() override
   { fix_num_length_and_dec(); return FALSE; }
   Item *copy_or_same(THD* thd) override;
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_sum_udf_decimal>(thd, this); }
 };
 
@@ -2104,7 +2108,7 @@ public:
   void print(String *str, enum_query_type query_type) override;
   bool change_context_processor(void *cntx) override
     { context= (Name_resolution_context *)cntx; return FALSE; }
-  Item *get_copy(THD *thd) override
+  Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_group_concat>(thd, this); }
   qsort_cmp2 get_comparator_function_for_distinct();
   qsort_cmp2 get_comparator_function_for_order_by();
