@@ -778,13 +778,12 @@ net_real_write(NET *net,const uchar *packet, size_t len)
 #ifdef MYSQL_SERVER
       if (global_system_variables.log_warnings > 3)
       {
-        my_printf_error(net->last_errno,
-                        "Could not write packet: fd: %lld  state: %d  "
-                        "errno: %d  vio_errno: %d  length: %ld",
-                        MYF(ME_ERROR_LOG),
-                        (longlong) vio_fd(net->vio), (int) net->vio->state,
-                        vio_errno(net->vio), net->last_errno, (ulong) (end-pos));
-        break;
+        sql_print_warning("Could not write packet: fd: %lld  state: %d  "
+                          "errno: %d  vio_errno: %d  length: %ld",
+                          MYF(ME_ERROR_LOG | ME_WARNING),
+                          (longlong) vio_fd(net->vio), (int) net->vio->state,
+                          vio_errno(net->vio), net->last_errno,
+                          (ulong) (end-pos));
       }
 #endif
       MYSQL_SERVER_my_error(net->last_errno, MYF(0));
@@ -1101,17 +1100,16 @@ retry:
 #ifdef MYSQL_SERVER
           if (global_system_variables.log_warnings > 3)
           {
-            my_printf_error(net->last_errno,
-                            "Could not read packet: fd: %lld  state: %d  "
-                            "remain: %u  errno: %d  vio_errno: %d  "
-                            "length: %lld",
-                            MYF(ME_ERROR_LOG),
-                            (longlong) vio_fd(net->vio), (int) net->vio->state,
-                            remain, vio_errno(net->vio), net->last_errno,
-                            (longlong) length);
+            /* Log things as a warning */
+            sql_print_warning("Could not read packet: fd: %lld  state: %d  "
+                              "read_length: %u  errno: %d  vio_errno: %d  "
+                              "length: %lld",
+                              (longlong) vio_fd(net->vio),
+                              (int) net->vio->state,
+                              remain, vio_errno(net->vio), net->last_errno,
+                              (longlong) length);
           }
-          else
-            my_error(net->last_errno, MYF(0));
+          my_error(net->last_errno, MYF(0));
 #endif /* MYSQL_SERVER */
 	  goto end;
 	}
