@@ -155,6 +155,24 @@ private:
   my_bool m_wsrep_on;
 };
 
+class wsrep_ignore_table
+{
+public:
+  wsrep_ignore_table(THD* thd)
+    : m_thd(thd)
+    , m_wsrep_ignore_table(thd->wsrep_ignore_table)
+  {
+    thd->wsrep_ignore_table= true;
+  }
+  ~wsrep_ignore_table()
+  {
+    m_thd->wsrep_ignore_table= m_wsrep_ignore_table;
+  }
+private:
+  THD* m_thd;
+  my_bool m_wsrep_ignore_table;
+};
+
 class thd_server_status
 {
 public:
@@ -739,6 +757,12 @@ int Wsrep_schema::store_view(THD* thd, const Wsrep_view& view)
   Wsrep_schema_impl::binlog_off binlog_off(thd);
   Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
 
+  if (trans_begin(thd, MYSQL_START_TRANS_OPT_READ_WRITE))
+  {
+    WSREP_ERROR("Failed to start transaction for store view");
+    goto out_not_started;
+  }
+
   /*
     Clean up cluster table and members table.
   */
@@ -832,7 +856,22 @@ int Wsrep_schema::store_view(THD* thd, const Wsrep_view& view)
 #endif /* WSREP_SCHEMA_MEMBERS_HISTORY */
   ret= 0;
  out:
+  if (ret)
+  {
+    trans_rollback_stmt(thd);
+    if (!trans_rollback(thd))
+    {
+      close_thread_tables(thd);
+    }
+  }
+  else if (trans_commit(thd))
+  {
+    ret= 1;
+    WSREP_ERROR("Failed to commit transaction for store view");
+  }
+  thd->release_transactional_locks();
 
+out_not_started:
   DBUG_RETURN(ret);
 }
 
@@ -1184,7 +1223,7 @@ int Wsrep_schema::remove_fragments(THD* thd,
   int ret= 0;
 
   WSREP_DEBUG("Removing %zu fragments", fragments.size());
-  Wsrep_schema_impl::wsrep_off  wsrep_off(thd);
+  Wsrep_schema_impl::wsrep_ignore_table wsrep_ignore_table(thd);
   Wsrep_schema_impl::binlog_off binlog_off(thd);
   Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
 
@@ -1244,23 +1283,16 @@ int Wsrep_schema::remove_fragments(THD* thd,
   DBUG_RETURN(ret);
 }
 
-int Wsrep_schema::replay_transaction(THD* orig_thd,
-                                     Relay_log_info* rli,
-                                     const wsrep::ws_meta& ws_meta,
-                                     const std::vector<wsrep::seqno>& fragments)
+static int replay_transaction(THD* thd,
+                              THD* orig_thd,
+                              Relay_log_info* rli,
+                              const wsrep::ws_meta& ws_meta,
+                              const std::vector<wsrep::seqno>& fragments)
 {
-  DBUG_ENTER("Wsrep_schema::replay_transaction");
-  DBUG_ASSERT(!fragments.empty());
-
-  THD thd(next_thread_id(), true);
-  thd.thread_stack= (orig_thd ? orig_thd->thread_stack :
-                     (char*) &thd);
-  wsrep_assign_from_threadvars(&thd);
-
-  Wsrep_schema_impl::wsrep_off  wsrep_off(&thd);
-  Wsrep_schema_impl::binlog_off binlog_off(&thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(&thd);
-  Wsrep_schema_impl::thd_context_switch thd_context_switch(orig_thd, &thd);
+  Wsrep_schema_impl::wsrep_off  wsrep_off(thd);
+  Wsrep_schema_impl::binlog_off binlog_off(thd);
+  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
+  Wsrep_schema_impl::thd_context_switch thd_context_switch(orig_thd, thd);
 
   int ret= 1;
   int error;
@@ -1272,12 +1304,12 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
   for (std::vector<wsrep::seqno>::const_iterator i= fragments.begin();
        i != fragments.end(); ++i)
   {
-    Wsrep_schema_impl::init_stmt(&thd);
-    if ((error= Wsrep_schema_impl::open_for_read(&thd, sr_table_str.c_str(), &frag_table_l)))
+    Wsrep_schema_impl::init_stmt(thd);
+    if ((error= Wsrep_schema_impl::open_for_read(thd, sr_table_str.c_str(), &frag_table_l)))
     {
       WSREP_WARN("Could not open SR table for read: %d", error);
-      Wsrep_schema_impl::finish_stmt(&thd);
-      DBUG_RETURN(1);
+      Wsrep_schema_impl::finish_stmt(thd);
+      return 1;
     }
     frag_table= frag_table_l.table;
 
@@ -1308,7 +1340,7 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
     frag_table->field[4]->val_str(&buf);
 
     {
-      Wsrep_schema_impl::thd_context_switch thd_context_switch(&thd, orig_thd);
+      Wsrep_schema_impl::thd_context_switch thd_context_switch(thd, orig_thd);
 
       ret= wsrep_apply_events(orig_thd, rli, buf.ptr(), buf.length());
       if (ret)
@@ -1319,17 +1351,18 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
     }
 
     Wsrep_schema_impl::end_index_scan(frag_table);
-    Wsrep_schema_impl::finish_stmt(&thd);
+    Wsrep_schema_impl::finish_stmt(thd);
 
-    Wsrep_schema_impl::init_stmt(&thd);
+    Wsrep_schema_impl::init_stmt(thd);
 
-    if ((error= Wsrep_schema_impl::open_for_write(&thd,
+    if ((error= Wsrep_schema_impl::open_for_write(thd,
                                                   sr_table_str.c_str(),
                                                   &frag_table_l)))
     {
       WSREP_WARN("Could not open SR table for write: %d", error);
-      Wsrep_schema_impl::finish_stmt(&thd);
-      DBUG_RETURN(1);
+      Wsrep_schema_impl::finish_stmt(thd);
+      ret= 1;
+      break;
     }
     frag_table= frag_table_l.table;
 
@@ -1355,88 +1388,108 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
       break;
     }
     Wsrep_schema_impl::end_index_scan(frag_table);
-    Wsrep_schema_impl::finish_stmt(&thd);
+    Wsrep_schema_impl::finish_stmt(thd);
     my_free(key);
     key= NULL;
   }
 
   if (key)
     my_free(key);
+
+  return ret;
+}
+
+int Wsrep_schema::replay_transaction(THD* orig_thd,
+                                     Relay_log_info* rli,
+                                     const wsrep::ws_meta& ws_meta,
+                                     const std::vector<wsrep::seqno>& fragments)
+{
+  DBUG_ENTER("Wsrep_schema::replay_transaction");
+  DBUG_ASSERT(!fragments.empty());
+
+  THD *thd= new THD(next_thread_id(), true);
+  if (!thd)
+  {
+    WSREP_WARN("Could not allocate memory for THD");
+    DBUG_RETURN(1);
+  }
+
+  thd->thread_stack= (orig_thd ? orig_thd->thread_stack : (char *) &thd);
+  wsrep_assign_from_threadvars(thd);
+
+  int ret= ::replay_transaction(thd, orig_thd, rli, ws_meta, fragments);
+
+  delete thd;
   DBUG_RETURN(ret);
 }
 
-int Wsrep_schema::recover_sr_transactions(THD *orig_thd)
+static int recover_sr_transactions(THD* storage_thd, THD* orig_thd)
 {
-  DBUG_ENTER("Wsrep_schema::recover_sr_transactions");
-  THD storage_thd(next_thread_id(), true);
-  storage_thd.thread_stack= (orig_thd ? orig_thd->thread_stack :
-                             (char*) &storage_thd);
-  wsrep_assign_from_threadvars(&storage_thd);
   TABLE* frag_table= 0;
   TABLE_LIST frag_table_l;
   TABLE* cluster_table= 0;
   TABLE_LIST cluster_table_l;
-  Wsrep_storage_service storage_service(&storage_thd);
-  Wsrep_schema_impl::binlog_off binlog_off(&storage_thd);
-  Wsrep_schema_impl::wsrep_off wsrep_off(&storage_thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(&storage_thd);
+  Wsrep_storage_service storage_service(storage_thd);
+  Wsrep_schema_impl::binlog_off binlog_off(storage_thd);
+  Wsrep_schema_impl::wsrep_off wsrep_off(storage_thd);
+  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(storage_thd);
   Wsrep_schema_impl::thd_context_switch thd_context_switch(orig_thd,
-                                                           &storage_thd);
+                                                           storage_thd);
   Wsrep_server_state& server_state(Wsrep_server_state::instance());
 
   int ret= 1;
   int error;
   wsrep::id cluster_id;
 
-  Wsrep_schema_impl::init_stmt(&storage_thd);
-  storage_thd.wsrep_skip_locking= FALSE;
-  if (Wsrep_schema_impl::open_for_read(&storage_thd, cluster_table_str.c_str(),
+  Wsrep_schema_impl::init_stmt(storage_thd);
+  storage_thd->wsrep_skip_locking= FALSE;
+  if (Wsrep_schema_impl::open_for_read(storage_thd, cluster_table_str.c_str(),
                                        &cluster_table_l))
   {
-    Wsrep_schema_impl::finish_stmt(&storage_thd);
-    DBUG_RETURN(1);
+    Wsrep_schema_impl::finish_stmt(storage_thd);
+    return 1;
   }
   cluster_table= cluster_table_l.table;
 
   if (Wsrep_schema_impl::init_for_scan(cluster_table))
   {
-    Wsrep_schema_impl::finish_stmt(&storage_thd);
-    DBUG_RETURN(1);
+    Wsrep_schema_impl::finish_stmt(storage_thd);
+    return 1;
   }
 
   if ((error= Wsrep_schema_impl::next_record(cluster_table)))
   {
     Wsrep_schema_impl::end_scan(cluster_table);
-    Wsrep_schema_impl::finish_stmt(&storage_thd);
-    trans_commit(&storage_thd);
+    Wsrep_schema_impl::finish_stmt(storage_thd);
+    trans_commit(storage_thd);
     if (error == HA_ERR_END_OF_FILE)
     {
       WSREP_INFO("Cluster table is empty, not recovering transactions");
-      DBUG_RETURN(0);
+      return 0;
     }
     else
     {
       WSREP_ERROR("Failed to read cluster table: %d", error);
-      DBUG_RETURN(1);
+      return 1;
     }
   }
 
   Wsrep_schema_impl::scan(cluster_table, 0, cluster_id);
   Wsrep_schema_impl::end_scan(cluster_table);
-  Wsrep_schema_impl::finish_stmt(&storage_thd);
+  Wsrep_schema_impl::finish_stmt(storage_thd);
 
   std::ostringstream os;
   os << cluster_id;
   WSREP_INFO("Recovered cluster id %s", os.str().c_str());
 
-  storage_thd.wsrep_skip_locking= TRUE;
-  Wsrep_schema_impl::init_stmt(&storage_thd);
+  storage_thd->wsrep_skip_locking= TRUE;
+  Wsrep_schema_impl::init_stmt(storage_thd);
 
   /*
     Open the table for reading and writing so that fragments without
     valid seqno can be deleted.
   */
-  if (Wsrep_schema_impl::open_for_write(&storage_thd, sr_table_str.c_str(),
+  if (Wsrep_schema_impl::open_for_write(storage_thd, sr_table_str.c_str(),
                                         &frag_table_l))
   {
     WSREP_ERROR("Failed to open SR table for write");
@@ -1492,7 +1545,7 @@ int Wsrep_schema::recover_sr_transactions(THD *orig_thd)
                                                          transaction_id)))
       {
         DBUG_ASSERT(wsrep::starts_transaction(flags));
-        applier = wsrep_create_streaming_applier(&storage_thd, "recovery");
+        applier = wsrep_create_streaming_applier(storage_thd, "recovery");
         server_state.start_streaming_applier(server_id, transaction_id,
                                              applier);
         applier->start_transaction(wsrep::ws_handle(transaction_id, 0),
@@ -1520,9 +1573,29 @@ int Wsrep_schema::recover_sr_transactions(THD *orig_thd)
     }
   }
   Wsrep_schema_impl::end_scan(frag_table);
-  Wsrep_schema_impl::finish_stmt(&storage_thd);
-  trans_commit(&storage_thd);
-  storage_thd.set_mysys_var(0);
+  Wsrep_schema_impl::finish_stmt(storage_thd);
+  trans_commit(storage_thd);
+  storage_thd->set_mysys_var(0);
 out:
+  return ret;
+}
+
+int Wsrep_schema::recover_sr_transactions(THD *orig_thd)
+{
+  DBUG_ENTER("Wsrep_schema::recover_sr_transactions");
+
+  THD *storage_thd= new THD(next_thread_id(), true);
+  if (!storage_thd)
+  {
+    WSREP_WARN("Could not allocate memory for THD");
+    DBUG_RETURN(1);
+  }
+  storage_thd->thread_stack=
+      (orig_thd ? orig_thd->thread_stack : (char *) &storage_thd);
+  wsrep_assign_from_threadvars(storage_thd);
+
+  int ret= ::recover_sr_transactions(storage_thd, orig_thd);
+
+  delete storage_thd;
   DBUG_RETURN(ret);
 }
