@@ -538,6 +538,7 @@ TRANSACTIONAL_TARGET void trx_free_at_shutdown(trx_t *trx)
 	DBUG_LOG("trx", "Free prepared: " << trx);
 	trx->state = TRX_STATE_NOT_STARTED;
 	ut_ad(!UT_LIST_GET_LEN(trx->lock.trx_locks));
+	ut_d(*trx->detailed_error = '\0');
 	trx->free();
 }
 
@@ -918,6 +919,7 @@ trx_start_low(
 
 #ifdef WITH_WSREP
 	trx->xid.null();
+	trx->wsrep = wsrep_on(trx->mysql_thd);
 #endif /* WITH_WSREP */
 
 	ut_a(ib_vector_is_empty(trx->autoinc_locks));
@@ -1491,6 +1493,8 @@ TRANSACTIONAL_INLINE inline void trx_t::commit_in_memory(const mtr_t *mtr)
     trx_finalize_for_fts(this, undo_no != 0);
 
 #ifdef WITH_WSREP
+  ut_ad(is_wsrep() == wsrep_on(mysql_thd));
+
   /* Serialization history has been written and the transaction is
   committed in memory, which makes this commit ordered. Release commit
   order critical section. */
@@ -2084,9 +2088,9 @@ static my_bool trx_recover_for_mysql_callback(rw_trx_hash_element_t *element,
 }
 
 
-static my_bool trx_recover_reset_callback(rw_trx_hash_element_t *element,
-  void*)
+static my_bool trx_recover_reset_callback(void *el, void*)
 {
+  rw_trx_hash_element_t *element= static_cast<rw_trx_hash_element_t*>(el);
   element->mutex.wr_lock();
   if (trx_t *trx= element->trx)
   {
@@ -2138,9 +2142,10 @@ struct trx_get_trx_by_xid_callback_arg
 };
 
 
-static my_bool trx_get_trx_by_xid_callback(rw_trx_hash_element_t *element,
-  trx_get_trx_by_xid_callback_arg *arg)
+static my_bool trx_get_trx_by_xid_callback(void *el, void *a)
 {
+  auto element= static_cast<rw_trx_hash_element_t*>(el);
+  auto arg= static_cast<trx_get_trx_by_xid_callback_arg*>(a);
   my_bool found= 0;
   element->mutex.wr_lock();
   if (trx_t *trx= element->trx)
