@@ -55,10 +55,6 @@ public:
   SPIDER_SHARE       *share;
   ulonglong          spider_thread_id;
   ulonglong          trx_conn_adjustment;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  ulonglong          trx_hs_r_conn_adjustment;
-  ulonglong          trx_hs_w_conn_adjustment;
-#endif
   uint               mem_calc_id;
   const char         *mem_calc_func_name;
   const char         *mem_calc_file_name;
@@ -71,16 +67,10 @@ public:
   char               *conn_keys_first_ptr;
   char               **conn_keys;
   SPIDER_CONN        **conns;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  char               **hs_r_conn_keys;
-  SPIDER_CONN        **hs_r_conns;
-  ulonglong          *hs_r_conn_ages;
-  char               **hs_w_conn_keys;
-  SPIDER_CONN        **hs_w_conns;
-  ulonglong          *hs_w_conn_ages;
-#endif
-  /* for active-standby mode */
+  /* array of indexes of active servers */
   uint               *conn_link_idx;
+  /* A bitmap indicating whether each active server have some higher
+  numbered server in the same "group" left to try (can fail over) */
   uchar              *conn_can_fo;
   void               **quick_targets;
   int                *need_mons;
@@ -94,6 +84,7 @@ public:
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   SPIDER_PARTITION_HANDLER *partition_handler;
 #endif
+  /* Whether this ha_spider is the owner of its wide_handler. */
   bool                wide_handler_owner = FALSE;
   SPIDER_WIDE_HANDLER *wide_handler = NULL;
 
@@ -162,7 +153,7 @@ public:
 #ifdef HANDLER_HAS_CAN_USE_FOR_AUTO_INC_INIT
   bool               auto_inc_temporary;
 #endif
-  int                bulk_size;
+  int                bulk_size= 0;
   int                direct_dup_insert;
   int                store_error_num;
   uint               dup_key_idx;
@@ -176,29 +167,16 @@ public:
   ulonglong          *db_request_id;
   uchar              *db_request_phase;
   uchar              *m_handler_opened;
+  /* ids for use in HANDLER command */
   uint               *m_handler_id;
+  /*
+    aliases for use in HANDLER command, in the format of t%5u on
+    m_handler_id. So for example, if m_handler_id is 3, then the
+    corresponding m_handler_cid is t00003
+  */
   char               **m_handler_cid;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uchar              *r_handler_opened;
-  uint               *r_handler_id;
-  uint               *r_handler_index;
-  uchar              *w_handler_opened;
-  uint               *w_handler_id;
-  uint               *w_handler_index;
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-  uchar              *do_hs_direct_update;
-  uint32             **hs_r_ret_fields;
-  uint32             **hs_w_ret_fields;
-  size_t             *hs_r_ret_fields_num;
-  size_t             *hs_w_ret_fields_num;
-  uchar              *tmp_column_bitmap;
-#endif
-#endif
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   bool               do_direct_update;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  bool               maybe_do_hs_direct_update;
-#endif
   uint               direct_update_kinds;
 #endif
   spider_index_rnd_init prev_index_rnd_init;
@@ -238,14 +216,14 @@ public:
   handler *clone(
     const char *name,
     MEM_ROOT *mem_root
-  );
+  ) override;
   const char **bas_ext() const;
   int open(
     const char* name,
     int mode,
     uint test_if_locked
-  );
-  int close();
+  ) override;
+  int close() override;
   int check_access_kind_for_connection(
     THD *thd,
     bool write_request
@@ -263,30 +241,30 @@ public:
     THD *thd,
     THR_LOCK_DATA **to,
     enum thr_lock_type lock_type
-  );
+  ) override;
   int external_lock(
     THD *thd,
     int lock_type
-  );
+  ) override;
   int start_stmt(
     THD *thd,
     thr_lock_type lock_type
-  );
-  int reset();
+  ) override;
+  int reset() override;
   int extra(
     enum ha_extra_function operation
-  );
+  ) override;
   int index_init(
     uint idx,
     bool sorted
-  );
+  ) override;
 #ifdef HA_CAN_BULK_ACCESS
   int pre_index_init(
     uint idx,
     bool sorted
   );
 #endif
-  int index_end();
+  int index_end() override;
 #ifdef HA_CAN_BULK_ACCESS
   int pre_index_end();
 #endif
@@ -295,36 +273,36 @@ public:
     const uchar *key,
     key_part_map keypart_map,
     enum ha_rkey_function find_flag
-  );
+  ) override;
   int index_read_last_map(
     uchar *buf,
     const uchar *key,
     key_part_map keypart_map
-  );
+  ) override;
   int index_next(
     uchar *buf
-  );
+  ) override;
   int index_prev(
     uchar *buf
-  );
+  ) override;
   int index_first(
     uchar *buf
-  );
+  ) override;
   int index_last(
     uchar *buf
-  );
+  ) override;
   int index_next_same(
     uchar *buf,
     const uchar *key,
     uint keylen
-  );
+  ) override;
   int read_range_first(
     const key_range *start_key,
     const key_range *end_key,
     bool eq_range,
     bool sorted
-  );
-  int read_range_next();
+  ) override;
+  int read_range_next() override;
   void reset_no_where_cond();
   bool check_no_where_cond();
 #ifdef HA_MRR_USE_DEFAULT_IMPL
@@ -336,7 +314,7 @@ public:
     uint *bufsz,
     uint *flags,
     Cost_estimate *cost
-  );
+  ) override;
   ha_rows multi_range_read_info(
     uint keyno,
     uint n_ranges,
@@ -345,17 +323,17 @@ public:
     uint *bufsz,
     uint *flags,
     Cost_estimate *cost
-  );
+  ) override;
   int multi_range_read_init(
     RANGE_SEQ_IF *seq,
     void *seq_init_param,
     uint n_ranges,
     uint mode,
     HANDLER_BUFFER *buf
-  );
+  ) override;
   int multi_range_read_next(
     range_id_t *range_info
-  );
+  ) override;
   int multi_range_read_next_first(
     range_id_t *range_info
   );
@@ -376,57 +354,57 @@ public:
 #endif
   int rnd_init(
     bool scan
-  );
+  ) override;
 #ifdef HA_CAN_BULK_ACCESS
   int pre_rnd_init(
     bool scan
   );
 #endif
-  int rnd_end();
+  int rnd_end() override;
 #ifdef HA_CAN_BULK_ACCESS
   int pre_rnd_end();
 #endif
   int rnd_next(
     uchar *buf
-  );
+  ) override;
   void position(
     const uchar *record
-  );
+  ) override;
   int rnd_pos(
     uchar *buf,
     uchar *pos
-  );
+  ) override;
   int cmp_ref(
     const uchar *ref1,
     const uchar *ref2
-  );
-  int ft_init();
-  void ft_end();
+  ) override;
+  int ft_init() override;
+  void ft_end() override;
   FT_INFO *ft_init_ext(
     uint flags,
     uint inx,
     String *key
-  );
+  ) override;
   int ft_read(
     uchar *buf
-  );
+  ) override;
   int pre_index_read_map(
     const uchar *key,
     key_part_map keypart_map,
     enum ha_rkey_function find_flag,
     bool use_parallel
-  );
-  int pre_index_first(bool use_parallel);
-  int pre_index_last(bool use_parallel);
+  ) override;
+  int pre_index_first(bool use_parallel) override;
+  int pre_index_last(bool use_parallel) override;
   int pre_index_read_last_map(
     const uchar *key,
     key_part_map keypart_map,
     bool use_parallel
-  );
+  ) override;
 #ifdef HA_MRR_USE_DEFAULT_IMPL
   int pre_multi_range_read_next(
     bool use_parallel
-  );
+  ) override;
 #else
   int pre_read_multi_range_first(
     KEY_MULTI_RANGE **found_range_p,
@@ -443,47 +421,47 @@ public:
     bool eq_range,
     bool sorted,
     bool use_parallel
-  );
-  int pre_ft_read(bool use_parallel);
-  int pre_rnd_next(bool use_parallel);
+  ) override;
+  int pre_ft_read(bool use_parallel) override;
+  int pre_rnd_next(bool use_parallel) override;
   int info(
     uint flag
-  );
+  ) override;
   ha_rows records_in_range(
     uint inx,
     const key_range *start_key,
     const key_range *end_key,
     page_range *pages
-  );
+  ) override;
   int check_crd();
-  int pre_records();
-  ha_rows records();
+  int pre_records() override;
+  ha_rows records() override;
 #ifdef HA_HAS_CHECKSUM_EXTENDED
-  int pre_calculate_checksum();
-  int calculate_checksum();
+  int pre_calculate_checksum() override;
+  int calculate_checksum() override;
 #endif
-  const char *table_type() const;
-  ulonglong table_flags() const;
+  const char *table_type() const override;
+  ulonglong table_flags() const override;
   ulong table_flags_for_partition();
   const char *index_type(
     uint key_number
-  );
+  ) override;
   ulong index_flags(
     uint idx,
     uint part,
     bool all_parts
-  ) const;
-  uint max_supported_record_length() const;
-  uint max_supported_keys() const;
-  uint max_supported_key_parts() const;
-  uint max_supported_key_length() const;
-  uint max_supported_key_part_length() const;
-  uint8 table_cache_type();
+  ) const override;
+  uint max_supported_record_length() const override;
+  uint max_supported_keys() const override;
+  uint max_supported_key_parts() const override;
+  uint max_supported_key_length() const override;
+  uint max_supported_key_part_length() const override;
+  uint8 table_cache_type() override;
 #ifdef HANDLER_HAS_NEED_INFO_FOR_AUTO_INC
-  bool need_info_for_auto_inc();
+  bool need_info_for_auto_inc() override;
 #endif
 #ifdef HANDLER_HAS_CAN_USE_FOR_AUTO_INC_INIT
-  bool can_use_for_auto_inc_init();
+  bool can_use_for_auto_inc_init() override;
 #endif
   int update_auto_increment();
   void get_auto_increment(
@@ -492,25 +470,25 @@ public:
     ulonglong nb_desired_values,
     ulonglong *first_value,
     ulonglong *nb_reserved_values
-  );
+  ) override;
   int reset_auto_increment(
     ulonglong value
-  );
-  void release_auto_increment();
+  ) override;
+  void release_auto_increment() override;
 #ifdef SPIDER_HANDLER_START_BULK_INSERT_HAS_FLAGS
   void start_bulk_insert(
     ha_rows rows,
     uint flags
-  );
+  ) override;
 #else
   void start_bulk_insert(
     ha_rows rows
   );
 #endif
-  int end_bulk_insert();
+  int end_bulk_insert() override;
   int write_row(
     const uchar *buf
-  );
+  ) override;
 #ifdef HA_CAN_BULK_ACCESS
   int pre_write_row(
     uchar *buf
@@ -522,21 +500,21 @@ public:
     bool hs_request
   );
 #endif
-  bool start_bulk_update();
+  bool start_bulk_update() override;
   int exec_bulk_update(
     ha_rows *dup_key_found
-  );
-  int end_bulk_update();
+  ) override;
+  int end_bulk_update() override;
 #ifdef SPIDER_UPDATE_ROW_HAS_CONST_NEW_DATA
   int bulk_update_row(
     const uchar *old_data,
     const uchar *new_data,
     ha_rows *dup_key_found
-  );
+  ) override;
   int update_row(
     const uchar *old_data,
     const uchar *new_data
-  );
+  ) override;
 #else
   int bulk_update_row(
     const uchar *old_data,
@@ -586,7 +564,7 @@ public:
 #ifdef SPIDER_MDEV_16246
   int direct_update_rows_init(
     List<Item> *update_fields
-  );
+  ) override;
 #else
   int direct_update_rows_init();
 #endif
@@ -647,7 +625,7 @@ public:
   int direct_update_rows(
     ha_rows *update_rows,
     ha_rows *found_row
-  );
+  ) override;
 #endif
 #ifdef HA_CAN_BULK_ACCESS
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
@@ -672,11 +650,11 @@ public:
 #endif
 #endif
 #endif
-  bool start_bulk_delete();
-  int end_bulk_delete();
+  bool start_bulk_delete() override;
+  int end_bulk_delete() override;
   int delete_row(
     const uchar *buf
-  );
+  ) override;
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   bool check_direct_delete_sql_part(
     st_select_lex *select_lex,
@@ -695,7 +673,7 @@ public:
     bool sorted
   );
 #else
-  int direct_delete_rows_init();
+  int direct_delete_rows_init() override;
 #endif
 #ifdef HA_CAN_BULK_ACCESS
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
@@ -727,7 +705,7 @@ public:
 #else
   int direct_delete_rows(
     ha_rows *delete_rows
-  );
+  ) override;
 #endif
 #ifdef HA_CAN_BULK_ACCESS
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
@@ -748,90 +726,90 @@ public:
 #endif
 #endif
 #endif
-  int delete_all_rows();
-  int truncate();
-  double scan_time();
+  int delete_all_rows() override;
+  int truncate() override;
+  double scan_time() override;
   double read_time(
     uint index,
     uint ranges,
     ha_rows rows
-  );
+  ) override;
 #ifdef HA_CAN_BULK_ACCESS
   void bulk_req_exec();
 #endif
-  const key_map *keys_to_use_for_scanning();
-  ha_rows estimate_rows_upper_bound();
+  const key_map *keys_to_use_for_scanning() override;
+  ha_rows estimate_rows_upper_bound() override;
   void print_error(
     int error,
     myf errflag
-  );
+  ) override;
   bool get_error_message(
     int error,
     String *buf
-  );
+  ) override;
   int create(
     const char *name,
     TABLE *form,
     HA_CREATE_INFO *info
-  );
+  ) override;
   void update_create_info(
     HA_CREATE_INFO* create_info
-  );
+  ) override;
   int rename_table(
     const char *from,
     const char *to
-  );
+  ) override;
   int delete_table(
     const char *name
-  );
-  bool is_crashed() const;
+  ) override;
+  bool is_crashed() const override;
 #ifdef SPIDER_HANDLER_AUTO_REPAIR_HAS_ERROR
-  bool auto_repair(int error) const;
+  bool auto_repair(int error) const override;
 #else
   bool auto_repair() const;
 #endif
   int disable_indexes(
     key_map map, bool persist
-  );
+  ) override;
   int enable_indexes(
     key_map map, bool persist
-  );
+  ) override;
   int check(
     THD* thd,
     HA_CHECK_OPT* check_opt
-  );
+  ) override;
   int repair(
     THD* thd,
     HA_CHECK_OPT* check_opt
-  );
+  ) override;
   bool check_and_repair(
     THD *thd
-  );
+  ) override;
   int analyze(
     THD* thd,
     HA_CHECK_OPT* check_opt
-  );
+  ) override;
   int optimize(
     THD* thd,
     HA_CHECK_OPT* check_opt
-  );
+  ) override;
   bool is_fatal_error(
     int error_num,
     uint flags
-  );
+  ) override;
   Field *field_exchange(
     Field *field
   );
   const COND *cond_push(
     const COND* cond
-  );
-  void cond_pop();
+  ) override;
+  void cond_pop() override;
   int info_push(
     uint info_type,
     void *info
-  );
+  ) override;
 #ifdef HANDLER_HAS_DIRECT_AGGREGATE
-  void return_record_by_parent();
+  void return_record_by_parent() override;
 #endif
   TABLE *get_table();
   void set_ft_discard_bitmap();
@@ -937,26 +915,6 @@ public:
   int reset_sql_sql(
     ulong sql_type
   );
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  int reset_hs_sql(
-    ulong sql_type
-  );
-  int reset_hs_keys(
-    ulong sql_type
-  );
-  int reset_hs_upds(
-    ulong sql_type
-  );
-  int reset_hs_strs(
-    ulong sql_type
-  );
-  int reset_hs_strs_pos(
-    ulong sql_type
-  );
-  int push_back_hs_upds(
-    SPIDER_HS_STRING_REF &info
-  );
-#endif
   int append_tmp_table_and_sql_for_bka(
     const key_range *start_key
   );
@@ -967,17 +925,9 @@ public:
   int reuse_union_table_and_sql_for_bka();
   int append_insert_sql_part();
   int append_update_sql_part();
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-  int append_increment_update_set_sql_part();
-#endif
-#endif
   int append_update_set_sql_part();
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   int append_direct_update_set_sql_part();
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  int append_direct_update_set_hs_part();
-#endif
 #endif
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   int append_dup_update_pushdown_sql_part(
@@ -1044,13 +994,6 @@ public:
     const key_range *end_key,
     ulong sql_type
   );
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  int append_key_where_hs_part(
-    const key_range *start_key,
-    const key_range *end_key,
-    ulong sql_type
-  );
-#endif
   int append_match_where_sql_part(
     ulong sql_type
   );
@@ -1105,13 +1048,6 @@ public:
     longlong limit,
     ulong sql_type
   );
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  int append_limit_hs_part(
-    longlong offset,
-    longlong limit,
-    ulong sql_type
-  );
-#endif
   int reappend_limit_sql_part(
     longlong offset,
     longlong limit,
@@ -1123,11 +1059,6 @@ public:
   int append_insert_values_sql_part(
     ulong sql_type
   );
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  int append_insert_values_hs_part(
-    ulong sql_type
-  );
-#endif
   int append_into_sql_part(
     ulong sql_type
   );
@@ -1196,9 +1127,6 @@ public:
   bool support_use_handler_sql(
     int use_handler
   );
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  bool support_bulk_access_hs() const;
-#endif
   int init_union_table_name_pos_sql();
   int set_union_table_name_pos_sql();
   int append_lock_tables_list();
