@@ -18,17 +18,12 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
 #include "sql_partition.h"
 #include "sql_select.h"
 #include "ha_partition.h"
-#endif
 #include "sql_common.h"
 #include <errmsg.h>
 #include "spd_err.h"
@@ -1047,7 +1042,7 @@ static int spider_prepare_init_scan(
     share->direct_order_limit);
   if (
     direct_order_limit &&
-    select_lex->explicit_limit &&
+    select_lex->limit_params.explicit_limit &&
     !(select_lex->options & OPTION_FOUND_ROWS) &&
     select_limit < direct_order_limit /* - offset_limit */
   ) {
@@ -1082,7 +1077,7 @@ static int spider_prepare_init_scan(
     result_list->internal_limit >= result_list->split_read ?
     result_list->split_read : result_list->internal_limit;
 
-  if (select_lex->explicit_limit)
+  if (select_lex->limit_params.explicit_limit)
   {
     result_list->internal_offset += offset_limit;
   } else {
@@ -1441,38 +1436,32 @@ group_by_handler *spider_create_group_by_handler(
       break;
   }
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   from = query->from;
   do {
     DBUG_PRINT("info",("spider from=%p", from));
     ++table_count;
+#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       DBUG_PRINT("info",("spider partition handler"));
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
       partition_info *part_info = from->table->part_info;
       uint bits = bitmap_bits_set(&part_info->read_partitions);
       DBUG_PRINT("info",("spider bits=%u", bits));
       if (bits != 1)
       {
         DBUG_PRINT("info",("spider using multiple partitions is not supported by this feature yet"));
-#else
-        DBUG_PRINT("info",("spider partition is not supported by this feature yet"));
-#endif
         DBUG_RETURN(NULL);
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
       }
-#endif
     }
-  } while ((from = from->next_local));
 #endif
+  } while ((from = from->next_local));
 
   if (!(table_holder= spider_create_table_holder(table_count)))
     DBUG_RETURN(NULL);
 
   table_idx = 0;
   from = query->from;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
   if (from->table->part_info)
   {
     partition_info *part_info = from->table->part_info;
@@ -1481,9 +1470,7 @@ group_by_handler *spider_create_group_by_handler(
     handler **handlers = partition->get_child_handlers();
     spider = (ha_spider *) handlers[part];
   } else {
-#endif
     spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   }
 #endif
   share = spider->share;
@@ -1507,7 +1494,7 @@ group_by_handler *spider_create_group_by_handler(
   }
   while ((from = from->next_local))
   {
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1516,9 +1503,7 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
@@ -1549,7 +1534,7 @@ group_by_handler *spider_create_group_by_handler(
 
   from = query->from;
   do {
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1558,9 +1543,7 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
@@ -1692,7 +1675,7 @@ group_by_handler *spider_create_group_by_handler(
     goto skip_free_table_holder;
 
   from = query->from;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
   if (from->table->part_info)
   {
     partition_info *part_info = from->table->part_info;
@@ -1701,9 +1684,7 @@ group_by_handler *spider_create_group_by_handler(
     handler **handlers = partition->get_child_handlers();
     spider = (ha_spider *) handlers[part];
   } else {
-#endif
     spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   }
 #endif
   share = spider->share;
@@ -1771,7 +1752,7 @@ group_by_handler *spider_create_group_by_handler(
   {
     fields->clear_conn_holder_from_conn();
 
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1780,9 +1761,7 @@ group_by_handler *spider_create_group_by_handler(
       handler **handlers = partition->get_child_handlers();
       spider = (ha_spider *) handlers[part];
     } else {
-#endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
