@@ -168,16 +168,6 @@ static int spider_parallel_search(THD *thd, SHOW_VAR *var, void *buff,
   DBUG_RETURN(0);
 }
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-static int spider_hs_result_free(THD *thd, SHOW_VAR *var, void *buff,
-                                 system_status_var *, enum_var_type)
-{
-  DBUG_ENTER("spider_hs_result_free");
-  spider_trx_status_var(thd, var, buff, &SPIDER_TRX::hs_result_free_count);
-  DBUG_RETURN(0);
-}
-#endif
-
 struct st_mysql_show_var spider_status_variables[] =
 {
   {"Spider_mon_table_cache_version",
@@ -207,13 +197,6 @@ struct st_mysql_show_var spider_status_variables[] =
     (char *) &spider_direct_aggregate, SHOW_FUNC},
   {"Spider_parallel_search",
     (char *) &spider_parallel_search, SHOW_FUNC},
-#endif
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#ifdef SPIDER_HAS_SHOW_SIMPLE_FUNC
-  {"Spider_hs_result_free", (char *) &spider_hs_result_free, SHOW_SIMPLE_FUNC},
-#else
-  {"Spider_hs_result_free", (char *) &spider_hs_result_free, SHOW_FUNC},
-#endif
 #endif
   {NullS, NullS, SHOW_LONG}
 };
@@ -575,25 +558,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_OVERRIDE_VALUE_FUNC(int, reset_sql_alloc)
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-/*
- -1 :fallback to default
-  0-:result free size for handlersocket
- */
-static MYSQL_THDVAR_LONGLONG(
-  hs_result_free_size, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Result free size for handlersocket", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  1048576, /* def */
-  -1, /* min */
-  9223372036854775807LL, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUN(longlong, hs_result_free_size)
-#endif
 
 /*
  -1 :fallback to default
@@ -1529,25 +1493,6 @@ static MYSQL_THDVAR_INT(
 
 SPIDER_THDVAR_VALUE_FUNC(double, ping_interval_at_trx_start)
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-/*
-  0 :always ping
-  1-:interval
- */
-static MYSQL_THDVAR_INT(
-  hs_ping_interval, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Ping interval for handlersocket", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  30, /* def */
-  0, /* min */
-  2147483647, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(double, hs_ping_interval)
-#endif
 
 /*
  -1 :fallback to default
@@ -1953,119 +1898,6 @@ static MYSQL_SYSVAR_LONGLONG(
 
 SPIDER_SYSVAR_OVERRIDE_VALUE_FUNC(longlong, udf_ct_bulk_insert_rows)
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-/*
-  0: no recycle
-  1: recycle in instance
-  2: recycle in thread
- */
-static MYSQL_THDVAR_UINT(
-  hs_r_conn_recycle_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Handlersocket connection recycle mode", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  2, /* def */
-  0, /* min */
-  2, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(uint, hs_r_conn_recycle_mode)
-
-/*
-  0: weak
-  1: strict
- */
-static MYSQL_THDVAR_UINT(
-  hs_r_conn_recycle_strict, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Strict handlersocket connection recycle", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  0, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(uint, hs_r_conn_recycle_strict)
-
-/*
-  0: no recycle
-  1: recycle in instance
-  2: recycle in thread
- */
-static MYSQL_THDVAR_UINT(
-  hs_w_conn_recycle_mode, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Handlersocket connection recycle mode", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  2, /* def */
-  0, /* min */
-  2, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(uint, hs_w_conn_recycle_mode)
-
-/*
-  0: weak
-  1: strict
- */
-static MYSQL_THDVAR_UINT(
-  hs_w_conn_recycle_strict, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Strict handlersocket connection recycle", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  0, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_VALUE_FUNC(uint, hs_w_conn_recycle_strict)
-
-/*
- -1 :fallback to default
-  0 :not use
-  1 :use handlersocket
- */
-static MYSQL_THDVAR_INT(
-  use_hs_read, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Use handlersocket for reading", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  -1, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUN(int, use_hs_read)
-
-/*
- -1 :fallback to default
-  0 :not use
-  1 :use handlersocket
- */
-static MYSQL_THDVAR_INT(
-  use_hs_write, /* name */
-  PLUGIN_VAR_RQCMDARG, /* opt */
-  "Use handlersocket for writing", /* comment */
-  NULL, /* check */
-  NULL, /* update */
-  0, /* def */
-  -1, /* min */
-  1, /* max */
-  0 /* blk */
-);
-
-SPIDER_THDVAR_OVERRIDE_VALUE_FUN(int, use_hs_write)
-#endif
 
 /*
  -1 :fallback to default
@@ -2737,9 +2569,6 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(semi_split_read_limit),
   MYSQL_SYSVAR(init_sql_alloc_size),
   MYSQL_SYSVAR(reset_sql_alloc),
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  MYSQL_SYSVAR(hs_result_free_size),
-#endif
   MYSQL_SYSVAR(multi_split_read),
   MYSQL_SYSVAR(max_order),
   MYSQL_SYSVAR(semi_trx_isolation),
@@ -2800,9 +2629,6 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(sts_bg_mode),
 #endif
   MYSQL_SYSVAR(ping_interval_at_trx_start),
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  MYSQL_SYSVAR(hs_ping_interval),
-#endif
   MYSQL_SYSVAR(auto_increment_mode),
   MYSQL_SYSVAR(same_server_link),
   MYSQL_SYSVAR(local_lock_table),
@@ -2825,14 +2651,6 @@ static struct st_mysql_sys_var* spider_system_variables[] = {
   MYSQL_SYSVAR(bka_mode),
   MYSQL_SYSVAR(udf_ct_bulk_insert_interval),
   MYSQL_SYSVAR(udf_ct_bulk_insert_rows),
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  MYSQL_SYSVAR(hs_r_conn_recycle_mode),
-  MYSQL_SYSVAR(hs_r_conn_recycle_strict),
-  MYSQL_SYSVAR(hs_w_conn_recycle_mode),
-  MYSQL_SYSVAR(hs_w_conn_recycle_strict),
-  MYSQL_SYSVAR(use_hs_read),
-  MYSQL_SYSVAR(use_hs_write),
-#endif
   MYSQL_SYSVAR(use_handler),
   MYSQL_SYSVAR(error_read_mode),
   MYSQL_SYSVAR(error_write_mode),
