@@ -244,7 +244,7 @@ class thread_pool_generic : public thread_pool
   unsigned int m_concurrency;
 
   /** True, if threadpool is being shutdown, false otherwise */
-  bool m_in_shutdown;
+  bool m_in_shutdown= false;
 
   /** Maintenance timer state : true = active(ON),false = inactive(OFF)*/
   enum class timer_state_t
@@ -304,11 +304,11 @@ class thread_pool_generic : public thread_pool
   }
 public:
   thread_pool_generic(int min_threads, int max_threads);
-  ~thread_pool_generic();
+  ~thread_pool_generic() override;
   void wait_begin() override;
   void wait_end() override;
   void submit_task(task *task) override;
-  virtual aio *create_native_aio(int max_io) override
+  aio *create_native_aio(int max_io) override
   {
 #ifdef _WIN32
     return create_win_aio(this, max_io);
@@ -436,13 +436,13 @@ public:
       m_task.wait();
     }
 
-    virtual ~timer_generic()
+    ~timer_generic() override
     {
       disarm();
     }
   };
   timer_generic m_maintenance_timer;
-  virtual timer* create_timer(callback_func func, void *data) override
+  timer* create_timer(callback_func func, void *data) override
   {
     return new timer_generic(func, data, this);
   }
@@ -722,9 +722,6 @@ static int  throttling_interval_ms(size_t n_threads,size_t concurrency)
 /* Create a new worker.*/
 bool thread_pool_generic::add_thread()
 {
-  if (m_thread_creation_pending.test_and_set())
-    return false;
-
   size_t n_threads = thread_count();
 
   if (n_threads >= m_max_threads)
@@ -750,6 +747,14 @@ bool thread_pool_generic::add_thread()
     }
   }
 
+  /* Check and set "thread creation pending" flag before creating the thread. We
+  reset the flag in thread_pool_generic::worker_main in new thread created. The
+  flag must be reset back in case we fail to create the thread. If this flag is
+  not reset all future attempt to create thread for this pool would not work as
+  we would return from here. */
+  if (m_thread_creation_pending.test_and_set())
+    return false;
+
   worker_data *thread_data = m_thread_data_cache.get();
   m_active_threads.push_back(thread_data);
   try
@@ -769,6 +774,7 @@ bool thread_pool_generic::add_thread()
         "current number of threads in pool %zu\n", e.what(), thread_count());
       warning_written = true;
     }
+    m_thread_creation_pending.clear();
     return false;
   }
   return true;
@@ -807,7 +813,6 @@ thread_pool_generic::thread_pool_generic(int min_threads, int max_threads) :
   m_wakeups(),
   m_spurious_wakeups(),
   m_timer_state(timer_state_t::ON),
-  m_in_shutdown(),
   m_timestamp(),
   m_long_tasks_count(),
   m_waiting_task_count(),
